{% extends "minimal/src/base.html" %}

{% block title %}{{ config.title }}: {{ post.title }}{% endblock %}

{% block content %}
      <nav>
        <ul>
          <li><a href="{{ home }}">Main index</a></li>
        </ul>
      </nav>

      <article class="h-entry">
        <h1 class="p-name">{{ post.title }}</h1>

        <p>Published by <a class="p-author h-card" href="{{ config.author.url }}">{{ config.author.name }}</a>
        on <time class="dt-published" datetime="{{ post.timestamp.isoformat() }}">{{ post.timestamp }}</time></p>

        <p class="p-summary"><small>{{ post.metadata.summary }}</small></p>

        <div class="e-content">
          {% autoescape false %}
          {{ render(post.content) }}
          {% endautoescape %}
        </div>
      </article>
{% endblock %}
